# Generated by roxygen2: do not edit by hand

S3method(.dpit_glm_key,Tweedie)
S3method(.dpit_glm_key,binomial)
S3method(.dpit_glm_key,default)
S3method(.dpit_glm_key,negbin)
S3method(.dpit_glm_key,poisson)
S3method(.dpit_zeroinfl_key,zeroinfl_default)
S3method(.dpit_zeroinfl_key,zeroinfl_negbin)
S3method(.dpit_zeroinfl_key,zeroinfl_poisson)
S3method(.gof_glm_key, poisson)
S3method(.gof_glm_key,binomial)
S3method(.gof_glm_key,default)
S3method(.gof_glm_key,negbin)
S3method(.gof_zeroinfl_key,default)
S3method(.gof_zeroinfl_key,negbin)
S3method(.gof_zeroinfl_key,poisson)
S3method(.quasi_plot_glm_key,binomial)
S3method(.quasi_plot_glm_key,default)
S3method(.quasi_plot_glm_key,nb)
S3method(.quasi_plot_glm_key,poisson)
S3method(.quasi_plot_zeroinfl_key,default)
S3method(.quasi_plot_zeroinfl_key,negbin)
S3method(.quasi_plot_zeroinfl_key,poisson)
S3method(dpit,default)
S3method(dpit,glm)
S3method(dpit,polr)
S3method(dpit,tobit)
S3method(dpit,vglm)
S3method(dpit,zeroinfl)
S3method(gof_disc,default)
S3method(gof_disc,glm)
S3method(gof_disc,polr)
S3method(gof_disc,zeroinfl)
S3method(ord_curve,default)
S3method(ord_curve,glm)
S3method(ord_curve,lm)
S3method(ord_curve,polr)
S3method(plot,dpit)
S3method(print,dpit)
S3method(print,summary.dpit)
S3method(quasi_plot,default)
S3method(quasi_plot,glm)
S3method(quasi_plot,polr)
S3method(quasi_plot,zeroinfl)
S3method(residuals,dpit)
S3method(summary,dpit)
export(dpit)
export(dpit_2pm)
export(dpit_bin)
export(dpit_nb)
export(dpit_ordi)
export(dpit_pois)
export(dpit_tobit)
export(dpit_tweedie)
export(dpit_znb)
export(dpit_zpois)
export(gof_disc)
export(ord_curve)
export(quasi_plot)
import(graphics)
import(np)
import(stats)
import(tweedie)
importFrom(MASS,gamma.dispersion)
importFrom(MASS,glm.nb)
importFrom(MASS,polr)
importFrom(MASS,rnegbin)
importFrom(graphics,abline)
importFrom(pscl,zeroinfl)
importFrom(stats,ecdf)
importFrom(stats,family)
importFrom(utils,combn)
importFrom(utils,modifyList)
