django_assessor
===============

Simple Assessment (Quiz) App with Django, Tastypie and ExtJS

LICENSE: AGPLv3

NOTE: Currently setup to use questions from practice exams for the CA BBS MFT 
certification exams. These are copyrighted and not for public use.

To use, you must create questions, choices and explanations manually. 

TODO:
- Create UI for question/choice/explanation creation
- Create UI for answer history
- Create UI for user authentication and authorization
- Create standalone install for Android
- Optimize AJAX calls, including LoadMask indicator
