#' @title Healthcare expenditure data
#' @description
#' Healthcare expenditure data set.
#' @source http://www.meps.ahrq.gov/mepsweb/
#' @format A data frame with 29784 rows and 29 variables:
#' \describe{
#'   \item{\code{EXP}}{the aggregate annual office based expenditure per participants, semicontinuous outcomes}
#'   \item{\code{AGE}}{Age}
#'   \item{\code{GENDER}}{1 if female}
#'   \item{\code{ASIAN}}{1 if Asian}
#'   \item{\code{BLACK}}{1 if Black}
#'   \item{\code{NORTHEAST}}{1 if Northeast}
#'   \item{\code{MIDWEST}}{1 if Midwest}
#'   \item{\code{SOUTH}}{1 if South}
#'   \item{\code{USC}}{1 if have usual source of care}
#'   \item{\code{COLLEGE}}{1 if colleage or higher degrees}
#'   \item{\code{HIGHSCH}}{1 if high school degree}
#'   \item{\code{MARRIED}}{1 if married}
#'   \item{\code{WIDIVSEP}}{1 if widowed or divorced or separated}
#'   \item{\code{FAMSIZE}}{Family Size}
#'   \item{\code{HINCOME}}{1 if high income}
#'   \item{\code{MINCOME}}{1 if middle income}
#'   \item{\code{LINCOME}}{1 if low income}
#'   \item{\code{NPOOR}}{1 if near poor}
#'   \item{\code{POOR}}{1 if poor}
#'   \item{\code{FAIR}}{1 if fair}
#'   \item{\code{GOOD}}{1 if good}
#'   \item{\code{VGOOD}}{1 if very good}
#'   \item{\code{MNHPOOR}}{1 if poor or fair mental health}
#'   \item{\code{ANYLIMIT}}{1 if any functional or activity limitation}
#'   \item{\code{unemployed}}{1 if unemployed at the beginning of 2006}
#'   \item{\code{EDUCHEALTH}}{1 if education, health and social services}
#'   \item{\code{PUBADMIN}}{1 if public administration}
#'   \item{\code{insured}}{1 if is insured at the beginning of the year 2006}
#'   \item{\code{MANAGEDCARE}}{if enrolled in an HMO or a gatekeeper plan}
#'}
"MEPS"
