#' @title LGPIF Data
#' @description
#' Data from the Wisconsin Local Government Property Insurance Fund (LGPIF).
#' The LGPIF was established to provide property insurance for local
#' government entities that include counties, cities, towns,
#' villages, school districts, fire departments, and other
#' miscellaneous entities, and is administered by the Wisconsin Office of the Insurance Commissioner. Properties covered under this fund include government buildings, vehicles, and equipment.
#' @source https://sites.google.com/a/wisc.edu/jed-frees/
#' @format A data frame with 5677 rows and 41 variables:
#' \describe{
#' \item{\code{PolicyNum}}{Policy number}
#' \item{\code{Year}}{Policy year}
#' \item{\code{ClaimBC}}{Total building and contents (BC) claims in the year}
#' \item{\code{ClaimIM}}{Total inland marine (IM) claims in the year (contractor’s equipment)}
#' \item{\code{ClaimPN}}{Total comprehensive claims from new motor vehicles in the year}
#' \item{\code{ClaimPO}}{Total comprehensive claims from old motor vehicles in the year}
#' \item{\code{ClaimCN}}{Total collision claims from new vehicles in the year}
#' \item{\code{ClaimCO}}{Total collision claims from old vehicles in the year}
#' \item{\code{TypeCity}}{Indicator for city entity}
#' \item{\code{TypeCounty}}{Indicator for county entity}
#' \item{\code{TypeMisc}}{Indicator for miscellaneous entity}
#' \item{\code{TypeSchool}}{Indicator for school entity}
#' \item{\code{TypeTown}}{Indicator for town entity}
#' \item{\code{TypeVillage}}{Indicator for village entity}
#' \item{\code{IsRC}}{Indicator for replacement cost (motor vehicles)}
#' \item{\code{CoverageBC}}{Log coverage amount for building and contents (in millions of dollars)}
#' \item{\code{lnDeductBC}}{Log deductible amount for building and contents}
#' \item{\code{NoClaimCreditBC}}{Indicator for no BC claims in prior year}
#' \item{\code{yAvgBC}}{Average BC claim amount}
#' \item{\code{FreqBC}}{Frequency of BC claims}
#' \item{\code{CoverageIM}}{Log coverage amount for inland marine (in millions of dollars)}
#' \item{\code{lnDeductIM}}{Log deductible amount for inland marine}
#' \item{\code{NoClaimCreditIM}}{Indicator for no IM claims in prior year}
#' \item{\code{yAvgIM}}{Average IM claim amount}
#' \item{\code{FreqIM}}{Frequency of IM claims}
#' \item{\code{CoveragePN}}{Log coverage amount for comprehensive new vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditPN}}{Indicator for no PN claims in prior year}
#' \item{\code{yAvgPN}}{Average PN claim amount}
#' \item{\code{FreqPN}}{Frequency of PN claims}
#' \item{\code{CoveragePO}}{Log coverage amount for comprehensive old vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditPO}}{Indicator for no PO claims in prior year}
#' \item{\code{yAvgPO}}{Average PO claim amount}
#' \item{\code{FreqPO}}{Frequency of PO claims}
#' \item{\code{CoverageCN}}{Log coverage amount for collision of new vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditCN}}{Indicator for no CN claims in prior year}
#' \item{\code{yAvgCN}}{Average CN claim amount}
#' \item{\code{FreqCN}}{Frequency of CN claims}
#' \item{\code{CoverageCO}}{Log coverage amount for collision of old vehicles (in millions of dollars)}
#' \item{\code{NoClaimCreditCO}}{Indicator for no CO claims in prior year}
#' \item{\code{yAvgCO}}{Average CO claim amount}
#' \item{\code{FreqCO}}{Frequency of CO claims}
#' }
#' @references Frees, E. W., Lee, G., & Yang, L. (2016). Multivariate frequency-severity regression models in insurance. Risks, 4(1), 4.
"LGPIF"

